@@ -19,17 +19,29 @@ import zpy
 log = logging.getLogger(__name__)
 
 
-def use_gpu() -> None:
+def use_gpu(device_type='CUDA', use_cpus=True) -> None:
     """ Use GPU for rendering. """
-    devices = list(
-        bpy.context.preferences.addons['cycles'].preferences.devices)
-    log.debug(f'Devices available {devices}')
-    prefs = bpy.context.preferences.addons['cycles'].preferences
-    prefs.compute_device_type = 'CUDA'
-    devices = prefs.get_devices()
-    for device in devices[0]:
-        device.use = True
-    log.debug(f'Devices available {devices}')
+    preferences = bpy.context.preferences
+    cycles_preferences = preferences.addons["cycles"].preferences
+    cuda_devices, opencl_devices = cycles_preferences.get_devices()
+
+    if device_type == "CUDA":
+        devices = cuda_devices
+    elif device_type == "OPENCL":
+        devices = opencl_devices
+    else:
+        raise RuntimeError("Unsupported device type")
+
+    for device in devices:
+        if device.type == "CPU":
+            device.use = use_cpus
+        else:
+            device.use = True
+            activated_gpus.append(device.name)
+
+    cycles_preferences.compute_device_type = device_type
+    bpy.context.scene.cycles.device = "GPU"
+    log.info(f'using devices {devices}')
 
 
 @gin.configurable
@@ -46,24 +58,6 @@ def set_seed(seed: int = 0) -> None:
     np.random.seed(seed)
 
 
-def set_log_levels(level: str = None) -> None:
-    """ Set logger levels for all zpy modules. """
-    if level is None:
-        log_level = logging.INFO
-    elif level == 'info':
-        log_level = logging.INFO
-    elif level == 'debug':
-        log_level = logging.DEBUG
-    elif level == 'warning':
-        log_level = logging.WARNING
-    else:
-        log.warning(f'Invalid log level {level}')
-        return
-    log.warning(f'Setting log level to {log_level}')
-    for logger_name in ['zpy', 'zpy_addon', 'zpy.savers', 'bpy.zpy_addon']:
-        logging.getLogger(logger_name).setLevel(log_level)
-
-
 @gin.configurable
 def step(
     num_steps: int = 16,
@@ -74,10 +68,11 @@ def step(
     """ Step logic helper for the scene. """
     assert num_steps is not None, 'Invalid num_steps'
     assert num_steps > 0, 'Invalid num_steps'
+    scene = zpy.blender.verify_blender_scene()
     step_idx = 0
     if framerate > 0:
-        start = bpy.context.scene.frame_start
-        stop = bpy.context.scene.frame_end
+        start = scene.frame_start
+        stop = scene.frame_end
         log.info(f'Animation enabled. Min frames: {start}. Max frames: {stop}')
     while step_idx < num_steps:
         log.info('-----------------------------------------')
@@ -87,8 +82,8 @@ def step(
         start_time = time.time()
         if framerate > 0:
             current_frame = start_frame + step_idx * framerate
-            bpy.context.scene.frame_set(current_frame)
-            log.info(f'Animation frame {bpy.context.scene.frame_current}')
+            scene.frame_set(current_frame)
+            log.info(f'Animation frame {scene.frame_current}')
         # # Update the step_idx for all RandomEvent and Animator instances
         # RandomEvent.step_idx = step_idx
         yield step_idx
@@ -121,6 +116,37 @@ def connect_debugger_vscode(timeout: int = 3) -> None:
             time.sleep(1)
 
 
+@gin.configurable
+def verify_view_layer(
+    view_layer_name: str = 'prod',
+) -> bpy.types.ViewLayer:
+    """ Get and set the view layer for a scene. """
+    scene = zpy.blender.verify_blender_scene()
+    view_layer = scene.view_layers.get(view_layer_name, None)
+    if view_layer is None:
+        log.info(f'Could not find view layer {view_layer_name}')
+        # Default behavior is to use last view layer in view layer list
+        view_layer = scene.view_layers[-1]
+    log.info(f'Setting view layer to {view_layer.name}')
+    bpy.context.window.view_layer = view_layer
+    return view_layer
+
+
+@gin.configurable
+def verify_blender_scene(
+    blender_scene_name: str = 'Prod',
+) -> bpy.types.Scene:
+    """ Get and set the main scene. """
+    scene = bpy.data.scenes.get(blender_scene_name, None)
+    if scene is None:
+        log.info(f'Could not find scene {blender_scene_name}')
+        # Default behavior is to use the first scene
+        scene = bpy.data.scenes[0]
+    log.info(f'Setting scene to {scene.name}')
+    bpy.context.window.scene = scene
+    return scene
+
+
 def parse_config(text_name: str = 'config') -> None:
     """ Load gin config for scene """
     _text = bpy.data.texts.get(text_name, None)
@@ -170,13 +196,14 @@ def refresh_blender_ui() -> None:
     """
     log.debug(f'Refreshing Blender UI.')
     bpy.ops.wm.redraw_timer(type='DRAW_WIN_SWAP', iterations=1)
-    bpy.context.view_layer.update()
+    view_layer = zpy.blender.verify_view_layer()
+    view_layer.update()
 
 
 def load_scene(
     path: Union[str, Path],
-    auto_execute_scripts : bool = True,
-    ) -> None:
+    auto_execute_scripts: bool = True,
+) -> None:
     """ Load a scene from a *.blend file. """
     # HACK: Clear out scene of cameras and lights
     clear_scene(['CAMERA', 'LIGHT'])
@@ -193,7 +220,8 @@ def load_scene(
     # Allow execution of scripts inside loaded scene
     if auto_execute_scripts:
         log.warning('Allowing .blend file to run scripts automatically')
-        log.warning('   this is unsafe for blend files from an untrusted source')
+        log.warning(
+            '   this is unsafe for blend files from an untrusted source')
         bpy.context.preferences.filepaths.use_scripts_auto_execute = auto_execute_scripts
 
 
@@ -246,9 +274,10 @@ def scene_information() -> Dict:
         _kwarg['default'] = param.default
         run_kwargs.append(_kwarg)
 
+    scene = zpy.blender.verify_blender_scene()
     _ = {
-        'name': bpy.context.scene.zpy_scene_name,
-        'version': bpy.context.scene.zpy_scene_version,
+        'name': scene.zpy_scene_name,
+        'version': scene.zpy_scene_version,
         'description': scene_doc,
         'run_kwargs': run_kwargs,
         'export_date':  time.strftime("%m%d%Y_%H%M_%S"),
@@ -258,49 +287,3 @@ def scene_information() -> Dict:
     }
     log.info(f'{_}')
     return _
-
-
-@gin.configurable
-def load_hdri(
-    path: Union[str, Path],
-    scale: Tuple[float] = (1.0, 1.0, 1.0),
-) -> None:
-    """ Load an HDRI from path.
-
-    Great source of HDRIs:
-
-        https://hdrihaven.com/
-
-    """
-    log.info(f'Loading HDRI at {path}')
-    path = zpy.files.verify_path(path, make=False)
-    world = bpy.context.scene.world
-    world.use_nodes = True
-    out_node = world.node_tree.nodes.get('World Output', None)
-    if out_node is None:
-        out_node = world.node_tree.nodes.new('World Output')
-    bg_node = world.node_tree.nodes.get('Background', None)
-    if bg_node is None:
-        bg_node = world.node_tree.nodes.new('Background')
-    env_node = world.node_tree.nodes.get('Environment Texture', None)
-    if env_node is None:
-        env_node = world.node_tree.nodes.new('ShaderNodeTexEnvironment')
-    env_node.image = bpy.data.images.load(str(path))
-    env_node.texture_mapping.scale = mathutils.Vector(scale)
-    # env_node.texture_mapping.rotation = mathutils.Vector(
-    #     (0,-math.radians(90), math.radians(180)))
-    # world.node_tree.links.new(env_node.inputs[0], tex_node.outputs[4])
-    world.node_tree.links.new(bg_node.inputs[0], env_node.outputs[0])
-    world.node_tree.links.new(out_node.inputs[0], bg_node.outputs[0])
-
-
-@gin.configurable
-def random_hdri(
-    asset_dir: Union[str, Path] = '$ASSETS/lib/hdris/hdri_maker_lib/04k_Library',
-) -> Path:
-    """ Generate a random HDRI from an asset path. """
-    asset_directory = zpy.files.verify_path(
-        asset_dir, make=False, check_dir=True)
-    hdri_path = random.choice([x for x in asset_directory.iterdir() if x.is_file()])
-    load_hdri(hdri_path)
-    return hdri_path
