@@ -16,6 +16,15 @@ import zpy
 log = logging.getLogger(__name__)
 
 
+def verify(obj: Union[str, bpy.types.Object], check_none=True) -> bpy.types.Object:
+    """ Return object given name or Object type object. """
+    if isinstance(obj, str):
+        obj = bpy.data.objects.get(obj)
+    if check_none and obj is None:
+        raise ValueError(f'Could not find object {obj}.')
+    return obj
+
+
 def load_blend_obj(
         name: str,
         path: Union[str, Path],
@@ -23,6 +32,7 @@ def load_blend_obj(
 ) -> bpy.types.Object:
     """ Load object from blend file. """
     path = zpy.files.verify_path(path, make=False)
+    scene = zpy.blender.verify_blender_scene()
     with bpy.data.libraries.load(str(path), link=link) as (data_from, data_to):
         for from_obj in data_from.objects:
             if from_obj.startswith(name):
@@ -30,41 +40,35 @@ def load_blend_obj(
                 data_to.objects.append(from_obj)
     # Copy objects over to the current scene
     for obj in data_to.objects:
-        bpy.context.scene.collection.objects.link(obj)
-    bpy.ops.file.find_missing_files(directory=str(path.parent / 'TEX'))
+        scene.collection.objects.link(obj)
+   for texture_folder_name in ['Textures', 'textures','TEX']:
+        texture_dir = path.parent / texture_folder_name
+        if texture_dir.exists():
+            bpy.ops.file.find_missing_files(directory=str(texture_dir))
+            break
     return bpy.data.objects[name]
 
 
 def select(obj: Union[bpy.types.Object, str]) -> None:
-    """ Delete an object. """
-    if isinstance(obj, str):
-        obj = bpy.data.objects[obj]
-    if obj is not None:
-        # TODO: This sometimes does not work due to context issues
-        log.debug(f'Selecting obj: {obj.name}')
-        bpy.ops.object.select_all(action='DESELECT')
-        bpy.context.view_layer.objects.active = obj
-        bpy.data.objects[obj.name].select_set(True)
-    else:
-        log.debug(f'Could not find object')
+    """ Select an object. """
+    obj = verify(obj)
+    view_layer = zpy.blender.verify_view_layer()
+    # TODO: This sometimes does not work due to context issues
+    log.debug(f'Selecting obj: {obj.name}')
+    bpy.ops.object.select_all(action='DESELECT')
+    view_layer.objects.active = obj
+    bpy.data.objects[obj.name].select_set(True, view_layer=view_layer)
 
 
 def delete_obj(obj: Union[bpy.types.Object, str]) -> None:
     """ Delete an object. """
-    if isinstance(obj, str):
-        obj = bpy.data.objects[obj]
-    if obj is not None:
-        log.debug(f'Removing obj: {obj.name}')
-        # Make sure selected object is the active object
-        bpy.ops.object.select_all(action='DESELECT')
-        obj.select_set(True)
-        bpy.ops.object.delete()
-        # bpy.data.objects.remove(obj, do_unlink=True)
-    else:
-        log.debug(f'Could not find object')
+    obj = verify(obj)
+    select(obj)
+    log.debug(f'Removing obj: {obj.name}')
+    bpy.ops.object.delete()
+    # bpy.data.objects.remove(obj, do_unlink=True)
 
 
-@gin.configurable
 def is_inside(
     point: mathutils.Vector,
     obj: bpy.types.Object,
@@ -86,6 +90,7 @@ def is_inside(
 
 def for_obj_in_selected_objs(context) -> bpy.types.Object:
     """ Safe iterable for selected objects. """
+    zpy.blender.verify_view_layer()
     for obj in context.selected_objects:
         # Only meshes or empty objects TODO: Why the empty objects
         if not (obj.type == 'MESH' or obj.type == 'EMPTY'):
@@ -109,7 +114,7 @@ def for_obj_in_collections(
 
 
 def toggle_hidden(
-    obj: bpy.types.Object,
+    obj: Union[bpy.types.Object, str],
     hidden: bool = True,
     filter_string: str = None,
 ) -> None:
@@ -118,9 +123,7 @@ def toggle_hidden(
     Optionally filter by a string in object name.
 
     """
-    if obj is None:
-        log.warning('Empty object given to toggle_hidden')
-        return
+    obj = verify(obj)
     if hasattr(obj, 'hide_render') and hasattr(obj, 'hide_viewport'):
         if (filter_string is None) or (filter_string in obj.name):
             log.debug(f'Hiding object {obj.name}')
@@ -157,13 +160,14 @@ def randomly_hide_within_collection(
 
 
 def segment(
-    obj: bpy.types.Object,
+    obj: Union[bpy.types.Object, str],
     name: str = 'default',
     color: Tuple[float] = None,
     as_category: bool = False,
     as_single: bool = False,
 ) -> None:
     """ Segment an object."""
+    obj = verify(obj)
     if color is None:
         color = zpy.color.random_color(output_style='frgb')
     obj.color = zpy.color.frgb_to_frgba(color)
@@ -191,15 +195,17 @@ def segment(
 
 
 def populate_vertex_colors(
-        obj: bpy.types.Object,
+        obj: Union[bpy.types.Object, str],
         color_rgba: Tuple[float],
         seg_type: str = 'instance',
 ) -> None:
     """Fill the given Vertex Color Layer with the color parameter values"""
+    obj = verify(obj)
     if not obj.type == 'MESH':
         log.warning(f'Object {obj.name} is not a mesh, has no vertices.')
         return
-    select(obj)
+    # TODO: Is this select needed?
+    # select(obj)
     # Remove any existing vertex color data
     if len(obj.data.sculpt_vertex_colors):
         for vcol in obj.data.sculpt_vertex_colors.keys():
@@ -221,9 +227,10 @@ def populate_vertex_colors(
 
 
 def random_position_within_constraints(
-    obj: bpy.types.Object
+    obj: Union[bpy.types.Object, str],
 ) -> None:
     """ Randomize position of object within constraints. """
+    obj = verify(obj)
     # Make sure object has constraints
     _constraints = obj.constraints.get('Limit Location', None)
     if _constraints is not None:
@@ -241,60 +248,78 @@ def random_position_within_constraints(
         )
 
 
+@gin.configurable
 def copy(
-    obj: bpy.types.Object,
+    obj: Union[bpy.types.Object, str],
     name: str = None,
     collection: bpy.types.Collection = None,
+    is_library_object: bool = False,
 ) -> bpy.types.Object:
     """ Create a copy of the object. """
+    obj = verify(obj)
     new_obj = bpy.data.objects.new(obj.name, obj.data)
     if name is not None:
         new_obj.name = name
     if collection is not None:
         collection.objects.link(new_obj)
+    else:
+        # Add to scene collection if no collection given
+        scene = zpy.blender.verify_blender_scene()
+        scene.collection.objects.link(obj)
+    # TODO: Library Overriding functions
+    if is_library_object:
+        log.warning(
+            f'Making mesh and material data local for obj {new_obj.name}')
+        new_obj.data.make_local()
+        for i in range(len(new_obj.material_slots)):
+            bpy.data.objects[new_obj.name].material_slots[i].material.make_local()
+        # Original object reference is lost if local copies are made
+        new_obj = bpy.data.objects[new_obj.name]
     return new_obj
 
 
 def translate(
-    obj: bpy.types.Object,
+    obj: Union[bpy.types.Object, str],
     translation: Tuple[float] = (0, 0, 0),
 ) -> None:
     """ Translate an object (in blender units). """
-    # select(obj)
-    # bpy.ops.transform.translate(value=translation)
-    # bpy.context.view_layer.update()
-    mat_trans = mathutils.Matrix.Translation(translation)
-    obj.matrix_world = mat_trans @ obj.matrix_world
+    obj = verify(obj)
+    log.debug(f'Translating object {obj.name} by {translation}')
+    log.debug(f'Before - obj.matrix_world\n{obj.matrix_world}')
+    select(obj)
+    bpy.ops.transform.translate(value=translation)
+    log.debug(f'After - obj.matrix_world\n{obj.matrix_world}')
+
 
 def rotate(
-    obj: bpy.types.Object,
+    obj: Union[bpy.types.Object, str],
     rotation: float = 0,
     axis: str = 'Z'
 ) -> None:
     """ Rotate an object (in radians) """
-    # select(obj)
-    # bpy.ops.transform.rotate(value=rotation, orient_axis=axis)
-    # bpy.context.view_layer.update()
-    mat_rot = mathutils.Matrix.Rotation(rotation, 4, axis)
-    obj.matrix_world = mat_rot @ obj.matrix_world
+    obj = verify(obj)
+    log.info(f'Rotating object {obj.name} by {rotation} radians around {axis} axis. ')
+    log.debug(f'Before - obj.matrix_world\n{obj.matrix_world}')
+    select(obj)
+    bpy.ops.transform.rotate(value=rotation, orient_axis=axis)
+    log.debug(f'After - obj.matrix_world\n{obj.matrix_world}')
 
 
 def scale(
-    obj: bpy.types.Object,
+    obj: Union[bpy.types.Object, str],
     scale: Tuple[float] = (1.0, 1.0, 1.0)
 ) -> None:
     """ Scale an object """
-    # select(obj)
-    # bpy.ops.transform.resize(value=scale)
-    # bpy.context.view_layer.update()
-    mag = scale[0] + scale[1] + scale[2]
-    norm_vector = (scale[0] / mag, scale[1] / mag, scale[2] / mag)
-    mat_scale = mathutils.Matrix.Scale(mag / 3.0, 4, norm_vector)
-    obj.matrix_world = mat_scale @ obj.matrix_world
+    obj = verify(obj)
+    log.info(f'Scaling object {obj.name} by {scale}')
+    log.debug(f'Before - obj.matrix_world\n{obj.matrix_world}')
+    select(obj)
+    bpy.ops.transform.resize(value=scale)
+    log.debug(f'After - obj.matrix_world\n{obj.matrix_world}')
 
 
 def jitter(
-    obj: bpy.types.Object,
+    obj: Union[bpy.types.Object, str],
     translate_range: Tuple[Tuple[float]] = (
         (-0.05, 0.05),
         (-0.05, 0.05),
@@ -312,6 +337,7 @@ def jitter(
     ),
 ) -> None:
     """ Apply random scale (blender units) and rotation (radians) to object """
+    obj = verify(obj)
     translate(obj,
               translation=(
                   random.uniform(translate_range[0][0], translate_range[0][1]),
@@ -336,19 +362,28 @@ def jitter(
               random.uniform(scale_range[1][0], scale_range[1][1]),
               random.uniform(scale_range[2][0], scale_range[2][1]),
           ))
+    zpy.blender.verify_view_layer()
     bpy.context.view_layer.update()
 
 
 _SAVED_POSES = {}
 
 
-def save_pose(obj: bpy.types.Object, pose_name: str) -> None:
+def save_pose(
+    obj: Union[bpy.types.Object, str],
+    pose_name: str,
+) -> None:
     """ Save a pose (rot and pos) to dict. """
+    obj = verify(obj)
     log.info(f'Saving pose {pose_name} based on object {obj.name}')
     _SAVED_POSES[pose_name] = obj.matrix_world.copy()
 
 
-def restore_pose(obj: bpy.types.Object, pose_name: str) -> None:
+def restore_pose(
+    obj: Union[bpy.types.Object, str],
+    pose_name: str,
+) -> None:
     """ Restore an object to a position. """
+    obj = verify(obj)
     log.info(f'Restoring pose {pose_name} to object {obj.name}')
     obj.matrix_world = _SAVED_POSES[pose_name]
