@@ -21,35 +21,24 @@ def registerSceneProperties():
         name="Scene Name",
         description="Name of the scene, must match data portal.",
         default="default",
-        # set=set_scene_name,
     )
     bpy.types.Scene.zpy_scene_version = bpy.props.StringProperty(
         name="Scene Version",
         description="Version of the scene, must match data portal.",
         default="0",
-        # set=set_scene_version,
     )
+    bpy.types.Scene.zpy_export_dir = bpy.props.StringProperty(
+        name='Export Directory Path',
+        description="Path to directory for exporting packaged zumo scenes.",
+        default=str(zpy.files.default_temp_path()),
+        subtype='DIR_PATH',
+    )
     bpy.types.Scene.zpy_export_path = bpy.props.StringProperty(
         name='Export Path',
-        description="Export path for packaged zumo scenes.",
+        description="Export path for this zumo scene.",
         default=str(zpy.files.default_temp_path()),
         subtype='DIR_PATH',
     )
-
-
-def set_scene_name(self, value) -> None:
-    """ Fix the scene name: all lowercase maximum 10 letters """
-    self.zpy_scene_name = value.lower()[:10]
-
-
-def set_scene_version(self, value) -> None:
-    """ Fix the scene version: integer > 0 """
-    try:
-        self['zpy_scene_version'] = abs(int(value))
-    except ValueError:
-        log.warning('Scene version must be a int')
-        self.zpy_scene_version = '0'
-
 
 class OpenExportDirOperator(bpy.types.Operator):
     """ Open file browser at export dir. """
@@ -61,7 +50,7 @@ class OpenExportDirOperator(bpy.types.Operator):
 
     def execute(self, context):
         zpy.files.open_folder_in_explorer(
-            context.scene.zpy_export_path, make=True)
+            context.scene.zpy_export_dir, make=True)
         return {'FINISHED'}
 
 
@@ -80,11 +69,6 @@ class CleanUpDirOperator(bpy.types.Operator):
             path=context.scene.zpy_export_path,
             exts=['.blend1', '.blend2', '.blend3'],
         )
-        # Remove any existing nfo files
-        zpy.files.remove_files_with_suffix(
-            path=context.scene.zpy_export_path,
-            exts=['.nfo'],
-        )
         # TODO: Scene based clean up collections and objects listings (in the text editor)
         # TODO: Remove the custom scene scripts that are not needed for staging (keep run, config, categories for now)
         return {'FINISHED'}
@@ -108,9 +92,12 @@ class ExportOperator(bpy.types.Operator):
 
         # Create export directory in the Blender filepath
         export_dir_name = f'{context.scene.zpy_scene_name}_v{context.scene.zpy_scene_version}'
-        export_path = Path(context.scene.zpy_export_path) / export_dir_name
+        export_path = Path(context.scene.zpy_export_dir) / export_dir_name
         zpy.files.verify_path(export_path, make=True)
 
+        # Set the scene export path
+        context.scene.zpy_export_path = str(export_path)
+
         # Find missing files before export
         log.info('Export Step 1 of 4: Checking for any missing files.')
         bpy.context.window_manager.progress_update(10)
@@ -123,6 +110,7 @@ class ExportOperator(bpy.types.Operator):
             bpy.ops.wm.save_as_mainfile(
                 filepath=str(export_path / 'main.blend'),
                 compress=False,
+                relative_remap=True,
             )
             log.info('Export Step 2 of 4: Packing files into .blend.')
             bpy.context.window_manager.progress_update(30)
@@ -133,6 +121,7 @@ class ExportOperator(bpy.types.Operator):
             bpy.ops.wm.save_as_mainfile(
                 filepath=str(export_path / 'main.blend'),
                 compress=False,
+                relative_remap=True,
             )
         except Exception as e:
             self.report({'ERROR'}, str(e))
@@ -142,12 +131,6 @@ class ExportOperator(bpy.types.Operator):
         log.info('Export Step 3 of 4: Saving meta-information.')
         bpy.context.window_manager.progress_update(70)
 
-        # Save nfo file with some meta information
-        save_time = time.strftime("%m%d%Y_%H%M_%S")
-        nfo_file = export_path / \
-            f'v{context.scene.zpy_scene_version}_{save_time}.nfo'
-        nfo_file.write_text('')
-
         # Output scene information in ZUMO_META
         zpy.files.write_json(
             export_path / 'ZUMO_META.json',
@@ -156,18 +139,18 @@ class ExportOperator(bpy.types.Operator):
 
         # TODO: Export glTF into zip directory
 
+        # Clean up scene before zipping up
+        bpy.ops.scene.zpy_cleanup_scene()
+
         # Zip up the exported directory for easy upload
         log.info('Export Step 4 of 4: Zipping up package.')
         bpy.context.window_manager.progress_update(90)
         zpy.files.zip_file(
             in_path=export_path,
-            zip_path=Path(context.scene.zpy_export_path) /
+            zip_path=Path(context.scene.zpy_export_dir) /
             f'{export_dir_name}.zip',
         )
 
-        # Clean scene after every export
-        bpy.ops.scene.zpy_cleanup_scene()
-
         log.info('Export Completed.')
         bpy.context.window_manager.progress_end()
         return {'FINISHED'}
@@ -175,7 +158,7 @@ class ExportOperator(bpy.types.Operator):
 
 class SCENE_PT_ExportPanel(bpy.types.Panel):
     """ UI for the addon that is visible in Blender. """
-    bl_idname="SCENE_PT_ExportPanel"
+    bl_idname = "SCENE_PT_ExportPanel"
     bl_space_type = "VIEW_3D"
     bl_region_type = "UI"
     bl_label = "Export"
@@ -198,7 +181,7 @@ class SCENE_PT_ExportPanel(bpy.types.Panel):
         row = layout.row()
         row.label(text="Export Path")
         row = layout.row()
-        row.prop(scene, "zpy_export_path", text="")
+        row.prop(scene, "zpy_export_dir", text="")
         row = layout.row()
         row.operator(
             'scene.zpy_open_export_dir',
